var NecessidadeDeComprasController = function (ScreenService, General, restEngine, templateManager, ZHPromise, CalculoDaNecessidadeDeComprasController, MessageService) {
    
    var self = this;
    var boAltNece;
    
    this.PARAM_COMPLEMENT_TO_CHECK_ALL_ROWS = "_CHECK_ALL_ROWS";
    
    this.voltar = function(widget){
        General.backOrCancel(widget);
    };
    
    this.back = function(widget) {
        
        ScreenService.closeSwipe();
        widget.getParent().activate();
        
    };
    
    this.setCheckedRows = function(widget){
        
        widget.updateCheckedRows();

    };
    
    this.defineNivelProduto = function(widgetProduto) {
        
        var mascaraProduto = getLocalVar('CDPICTPROD');
        var fieldProduto = widgetProduto.getField('NMPRODUTO');
        
        mascaraProduto = mascaraProduto.split('.');
        var nivelProduto = mascaraProduto.length;
        
        fieldProduto.dataSourceFilter.forEach(function(filtro) {
            if(filtro.name == 'CDNVPRODUTO')
                filtro.value = nivelProduto;
        });
        
    };
   
    
    /**
     * Cria um parâmetro no templateManger e atribui o valor recebido
     * como parâmetro.
     * 
     * @param {string} parameterName - nome do parâmetro que será
     * criado no templateManager.
     * 
     * @param {*} value - valor que será atribuido ao parâmetro criado
     * no templateManager.
    */
    this.setParameterOnTemplateManager = function(parameterName, value){
        templateManager[parameterName] = value;
    };
    
    /**
     * Marca todas as linhas da widget recebida
     * como parâmetro.
     * 
     * @param {object} widget - widget que terá
     * todas as linhas marcadas.
    */
    this.checkAllRowsOnWidget = function(widget){
        
        var widgetDataSource = widget.dataSource;
        
        widgetDataSource.addCheckedRows(widgetDataSource.data);
        widgetDataSource.updateCheckedRows();

    };
    
    function insereProduto(rowProduto, produtosNecessidade) {
    
        produtosNecessidade.push(rowProduto);
        
    }
    
    this.openSwipeProduto = function(widget){
        var widgetProduto = widget.container.getWidget("insereProduto");
        
        ScreenService.openSwipe(widgetProduto); 
        widgetProduto.newRow();
        widgetProduto.currentRow.__is_new = true;
        widgetProduto.activate();
    }; 
    
    function checkProdAtivo(row) {
        var deferred = ZHPromise.defer();
        
        General.callRestEngine('cmp_necessidadeCompras#/checkProdAtivo','Row',row).then(function(calback) {
            if(calback.dataset.resposta.length != 0){
                if(calback.dataset.resposta[0].IDPRODATIV !== 'A'){
                    deferred.reject(ScreenService.i18n("O produto está inativo para a unidade."));
                } else if(calback.dataset.resposta[0].IDPRODATIVPR !== 'S'){
                    deferred.reject(ScreenService.i18n("O produto está inativo!"));
                } else {
                    deferred.resolve();
                }
            } else {
                deferred.reject(ScreenService.i18n("O produto está inativo!"));
            }
        });
        
        return deferred;
    }
    
    function checkProdNeceComp(rowProduto, produtosNecessidade) {

        var cdproduto = rowProduto.CDPRODUTO;
        var dtentrneccmp = rowProduto.DTENTRNECCMP;
        
        var existeProduto = produtosNecessidade.filter(function(produto) {
            return cdproduto === produto.CDPRODUTO && dtentrneccmp.substr(0,10) === produto.DTENTRNECCMP.substr(0,10);
        });
        
        if(existeProduto.length) {
            throw ScreenService.i18n("O Produto já foi incluído para a data informada!");
        }

    }
    
    function validaData(dtini, dtfim, dtentrega) {
        
        dtini = moment(dtini, 'DD/MM/YYYY');
        dtfim = moment(dtfim, 'DD/MM/YYYY');
        dtentrega = moment(dtentrega, 'DD/MM/YYYY');
        
        return dtentrega.isBetween(dtini, dtfim);
        
    }
    
    this.salvaProduto = function(widgetProduto){
    
        var widgetNecessidade = widgetProduto.container.getWidget('gridNeceComp');
        var produtosNecessidade = widgetNecessidade.dataSource.data;
        var rowProduto = widgetProduto.currentRow;
        
        rowProduto.CDFILIAL = widgetNecessidade.CDFILIAL;
        rowProduto.NRNECECOMP = widgetNecessidade.NECECOMP;
        rowProduto.DTPRIUTIPROD = rowProduto.DTENTRNECCMP;
        
        if(widgetProduto.isValid() && validaData(widgetNecessidade.DTINI, widgetNecessidade.DTFIM, rowProduto.DTENTRNECCMP)) {
            checkProdAtivo(rowProduto).then(function() {
                
                checkProdNeceComp(rowProduto, produtosNecessidade);
                
                insereProduto(rowProduto, produtosNecessidade);
                
                ScreenService.closeSwipe();
                widgetNecessidade.activate();
                widgetNecessidade.dataSource.addCheckedRows(produtosNecessidade);
                widgetNecessidade.updateCheckedRows();
                
            }).catch(function(error) {
                ScreenService.showMessage(error);
            });
        } else {
            MessageService.showMessageByCode("DT_FORA_PER");
        }
    };
    
    this.salvaQuantidade = function(widgetNecessidade) {
        widgetNecessidade.commitInlineEdit();
        widgetNecessidade.ALTERADO = true;
    };
    
    /**
     * Verifica se o parâmetro, referente a widget recebida como
     * parâmetro, no templateManager permite que todas as linhas da
     * widget sejam marcadas, se permitir, todas as linhas são marcadas
     * e logo após, o parâmetro é definido como "false", pois pela regra
     * de negócio, está rotina é realizada apenas uma vez.
     * 
     * @param {object} widget - widget que terá suas linhas marcadas.
    */
    this.checkAllWidgetRowsByTemplateManagerParam = function(widget){
        
        var parameterName = widget.name + this.PARAM_COMPLEMENT_TO_CHECK_ALL_ROWS;
        
        if(Boolean(templateManager[parameterName])){
            
            this.checkAllRowsOnWidget(widget);
            this.setParameterOnTemplateManager(parameterName, false);
            
        }    
        
    };
    
    /**
     * define "true" para os parâmetros que indicam se as widgets
     * de detalhes terão todas as linhas selecionadas.
    */
    this.defineParamToCheckAllDetailsWidgetsRows = function(){
        
        var widget =  this.getWidgetOnCurrentContainerByName("gridNeceComp");
        var parameterName = widget.name + this.PARAM_COMPLEMENT_TO_CHECK_ALL_ROWS;
        this.setParameterOnTemplateManager(parameterName, true);

    };
    
    /**
     * Retorna a widget com o nome recebido como parâmetro do container
     * ativo.
     * 
     * @param {string} widgetName - nome da widget a ser retornada.
     * 
     * @param {object} widget encontrada.
    */
    this.getWidgetOnCurrentContainerByName = function(widgetName){
        return templateManager.container.getWidget(widgetName);
    };
    
    this.openDetalhesProduto = function(widget){
        ScreenService.openSwipe(widget.container.getWidget("detalhesProduto"));
        if(widget.container.getWidget("detalhesProduto").dataSourceFilter[3] == undefined){
            widget.container.getWidget("detalhesProduto").dataSourceFilter.push(    			
                    {
        				'name': 'CDPRODNECCMP',
        				'operator': '=',
        				'value': widget.currentRow.CDARVPROD
        			}
        	);
        }else{
            widget.container.getWidget("detalhesProduto").dataSourceFilter.pop();
            widget.container.getWidget("detalhesProduto").dataSourceFilter.push(    			
                    {
        				'name': 'CDPRODNECCMP',
        				'operator': '=',
        				'value': widget.currentRow.CDARVPROD
        			}
        	);
        }
        widget.container.getWidget("detalhesProduto").reload();
        widget.container.getWidget("detalhesProduto").activate();
    };
    
    this.voltarCalculo = function(widget) {
        
        var widgetData = widget.dataSource.data;
        var calculationWindowName = "cmp_necessidadeCompras#calculo_da_necessidade_de_compras";
        
        this.defineParamToCheckAllDetailsWidgetsRows();
        
        if(widgetData.length){
            
            var row = [widgetData[0].NRNECECOMP];
            
            General.callRestEngine('cmp_necessidadeCompras#/deleteNececomp','Row',row).then(function(calback) {
                ScreenService.openWindow(calculationWindowName).then(function() {
                    delete templateManager.containers['cmp_necessidadeCompras#necessidade_de_compras'];
                });
            });
            
        } else {
            ScreenService.openWindow(calculationWindowName).then(function() {
                delete templateManager.containers['cmp_necessidadeCompras#necessidade_de_compras'];
            });
        }
        
	};
    
    this.gerarSolicitacao = function(widget){
        
        var produtosSelecionados = widget.getCheckedRows();
        
        if(produtosSelecionados.length > 0 ) {
            ScreenService.confirmMessage('Deseja gerar a solicitação?', 'question', function(success) {
                
                var extra = CalculoDaNecessidadeDeComprasController.getExtra();
                var nececomp = widget.dataSource.data[0].NRNECECOMP;
                var msg = ScreenService.i18n('Cálculo da Necessidade de Compras-Extra');
                var filnececomp = widget.CDFILIAL;
                var allProducts = widget.dataSource.data;
                var row = {'PRODUTOS': allProducts, 'NRNECECOMP': nececomp, 'CDFILNECECMP': filnececomp, 'EXTRA': extra, 'MENSAGEM': msg};
               
               General.callRestEngine('cmp_necessidadeCompras#/GetPrxSoliComp','Row',row).then(function(response) {
                   self.openPreviaCalculoNecessidadeCompras(nececomp);
                   MessageService.showMessageByCode("SOLCMP_SUCCESS");
                }.bind()).catch(function(error) {
                    MessageService.showMessageByCode("ERROR_FOUND", "ERROR");
                });
            });
        } else{
            MessageService.showMessageByCode("NO_DATA_GER_SOL");
        }
        
    };
    
    this.setVisibilidade = function(args){
        
       var widgetNeceComp = args.owner.getWidget('gridNeceComp');
       var parasupri;
       
        General.callRestEngine('cmp_necessidadeCompras#/retornaParasupriNececomp','Row', []).then(function(response) {
            
            parasupri = response.dataset.parasupri.length ? response.dataset.parasupri[0] : false;
            
            if(parasupri && parasupri.IDBLOQALTNEC == 'S'){
                widgetNeceComp.getField('QTPRCNVCOMPR').inlineEditable = false;
                widgetNeceComp.getAction('incluir').isVisible = false;
            }else{
                widgetNeceComp.getField('QTPRCNVCOMPR').inlineEditable = true;
                widgetNeceComp.getAction('incluir').isVisible = true;
            }

        }).catch(function(error) {
            MessageService.showMessageByCode("ERROR_FOUND", "ERROR");
        });
        
    };
    
    this.openPreviaCalculoNecessidadeCompras = function(numeroPrevia) {
        ScreenService.openWindow('cmp_necessidadeCompras#cmp03800_previa_do_calculo_da_necessidade_de_compras').then(function() {
            templateManager.container.getWidget('FILTRO').getField('NRNECECOMP').defaultValue = numeroPrevia;
        });
    };
    
};

Configuration(function (ContextRegister) {
	ContextRegister.register('NecessidadeDeComprasController', NecessidadeDeComprasController);
});