var CompraSimplificadaController = function (CompraSimplificadaService, ScreenService, General, restEngine, templateManager, Utilities) {
    
    const serviceCompraSimplificada = CompraSimplificadaService;
    
    let widgetCompra;
    let widgetUnidades;
    let widgetProdutos;
    let widgetAddFiltro;
    let widgetCalcNeceCompra;
    
    let parasupri;
    let centralCompras;
    let parcmpsup;
    let paramgeral;
    
    const messages = {
        "defaulNInf": "Parâmetros default de consumo devem ser informados.",
        "questionCalcPrevisao": "Deseja calcular a previsão de consumo no período entre as datas de processamento de cada unidade e o dia ",
        "confirmCalculo": "Confirma o cálculo?",
        "validaAcrescimo": "Acréscimo deve ser menor que 10000%",
        "exceptTempoLimite": "Tempo limite de conexão excedido."
    };
    
    this.setWidgets = function() {
        widgetCompra = templateManager.container.getWidget('compra_simplificada');
        widgetUnidades = templateManager.container.getWidget('filial_swipe');
        widgetProdutos = templateManager.container.getWidget('produto_swipe');
        widgetAddFiltro = templateManager.container.getWidget('adiciona_filtro_swipe');
        widgetCalcNeceCompra = templateManager.container.getWidget('opcoes_necessidadecompras_swipe');   
    };
    
    this.filterUnidadesProdutos = function() {
        widgetUnidades.dataSourceFilter = [
    	    {"name": "P_NRFILTROCOMPSFDA", "operator": "=", "value": widgetCompra.currentRow.NRFILTROCOMPSFDA}
    	];
    	
    	widgetProdutos.dataSourceFilter = [
    	    {"name": "P_NRFILTROCOMPSFDA", "operator": "=", "value": widgetCompra.currentRow.NRFILTROCOMPSFDA}
    	];
    	
    	widgetUnidades.reload();
    	widgetProdutos.reload();
    	
    	widgetUnidades.activate();
    };
    
    this.backCompraSimplificada = function() {
        ScreenService.closeSwipe();
        widgetCompra.activate();
    };
    
    this.backCancelNewFilter = function() {
        ScreenService.closePopup();
        widgetCompra.activate();
    };
    
    this.newFiltro = function() {
        widgetAddFiltro.newRow();
        ScreenService.openPopup(widgetAddFiltro).then(function() {
            widgetAddFiltro.getField('DTENTREGA').defaultValue.start = moment().format('DD/MM/YYYY');
            widgetAddFiltro.getField('DTENTREGA').defaultValue.end = moment().format('DD/MM/YYYY');
            widgetAddFiltro.getField('DTENTREGA').readOnly = true;
        });
    };
    
    this.saveFilter = function() {
        const filterRow = widgetAddFiltro.currentRow;
        
        serviceCompraSimplificada.saveFilterCompraSimplificada(filterRow).then(function(callback) {
            widgetAddFiltro.currentRow = {};
            widgetCompra.reload();
        }, function(err) {
            ScreenService.showMessage(err.data.error);
        });
    };
    
    this.changeNivelProduto = function() {
        const fieldProduto = widgetAddFiltro.getField('CDPRODUTO');
        const row = widgetAddFiltro.currentRow;
        
        fieldProduto.readOnly = row.CDNVPRODUTO ? false : true;
        fieldProduto.clearValue(fieldProduto);
    };
    
    this.setFilterProdutos = function(){
        const fieldProduto = widgetAddFiltro.getField('CDPRODUTO');
        const row = widgetAddFiltro.currentRow;

        fieldProduto.dataSourceFilter = [
            {"name": "P_CDNVPRODUTO", "operator": "=", "value": row.CDNVPRODUTO},
		    {
                "operator": "ORDER_BY",
                "name": [
                    {
                        "name": "CDARVPROD",
                        "direction": "ASC"
                    }
                ]
            }
        ];
        fieldProduto.reload();
    };
    
    this.deleteCompraSimplificada = function() {
        ScreenService.confirmMessage('Deletar registro?', 'question', function(success) {
            let compraArray = [];
            
            widgetCompra.dataSource.data.forEach(function(row) {
                if (row.__isSelected) {
                    compraArray.push(row);
                }
            });
            
            serviceCompraSimplificada.deleteFilterCompraSimplificada(compraArray).then(function(callback) {
                widgetCompra.unSelectAll();
                widgetCompra.reload();
            }, function(err) {
                ScreenService.showMessage(err.data.error);
            });
        });
    };
    
    function getParametrosNecessidadeCompra() {
        serviceCompraSimplificada.getParametrosNecessidadeCompra(widgetCompra.currentRow).then(function(callback) {
            parasupri       = callback.dataset.parasupri;
            parcmpsup       = callback.dataset.parcmpsup;
            paramgeral      = callback.dataset.paramgeral;
            centralCompras  = callback.dataset.central_compras;
        }, function(error) {
            ScreenService.showMessage(error.data.error);
        });
    }
    
    this.openWidgetOpcoesCalculo = function() {
        ScreenService.openPopup(widgetCalcNeceCompra).then(function() {
            getParametrosNecessidadeCompra(); 
        });
    };
    
    function validaAcrecimos(row) {
        if (row.PEACRESCIMO < 10000.00) {
            return true;
        } else {
            ScreenService.showMessage(messages.validaAcrescimo);
            return false;
        }
    }
    
    this.confirmaCalculo = function() {
        const idutcardpadr = parcmpsup.IDUTCARDPADR;
        
        let row = widgetCalcNeceCompra.currentRow;
            
        row.PEACRESCIMO = row.PEACRESCIMO ? row.PEACRESCIMO : 0;
        if(widgetCalcNeceCompra.isValid() && validaAcrecimos(row)) {
            ScreenService.confirmMessage(messages.confirmCalculo, 'question', function(success) {
                /*if(idutcardpadr == 'D' || idutcardpadr == 'S'){
                    
                    General.callRestEngine('cmp_necessidadeCompras#/previsoes','Row',row).then(function(calback) {
                        var resposta = calback.dataset.resposta;
                        
                        if(resposta.length != 0 ){
    
                            widget.container.getWidget('previsoesbloqueadas').dataSource.data = resposta;
                            widget.container.getWidget('previsoesbloqueadas').reload();
                            ScreenService.openPopup(widget.container.getWidget('previsoesbloqueadas'));
                           
                        } else {
                            self.verificaSolicitacao(widget, row);
                        }
                    }).catch(function(error) {
                        ScreenService.showMessage(error.data ? error.data.error : messages.exceptTempoLimite, 'error', 'error');
                    });
                }else {
                    self.verificaSolicitacao(widget, row);
                }   */ 
             
            });
        }
    };
    
    
    
    /*
     *
     * To Do
     * 
     *
     *
     *
     */
     
     /*this.validaAcrecimos = function(field){
        var mensagem = "Acréscimo deve ser menor que 10000%";
        
        if (field < 10000.00){
            return true;
        } else {
            ScreenService.showMessage(mensagem);
            return false;
        }
    };
    
    field.widget.getField('PEACRESCIMO').readOnly = field.widget.currentRow.IDPERACRNEC == 'N';*/ 
};

Configuration(function (ContextRegister) {
	ContextRegister.register('CompraSimplificadaController', CompraSimplificadaController);
});